@@ -1,19 +1,17 @@
-use js_sys::Reflect;
+use maplit::{btreemap, hashmap, hashset};
 use serde::de::DeserializeOwned;
 use serde::ser::Error as SerError;
 use serde::{Deserialize, Serialize};
 use serde_wasm_bindgen::{from_value, to_value, Error, Serializer};
+use std::collections::BTreeMap;
 use std::fmt::Debug;
-use std::{
-    collections::{BTreeMap, HashMap, HashSet},
-    hash::Hash,
-};
+use std::hash::Hash;
 use wasm_bindgen::{JsCast, JsValue};
 use wasm_bindgen_test::*;
 
 wasm_bindgen_test_configure!(run_in_browser);
 
-fn test<L, R>(lhs: L, rhs: R)
+fn test_via_into<L, R>(lhs: L, rhs: R)
 where
     L: Serialize + DeserializeOwned + PartialEq + Debug,
     R: Into<JsValue>,
@@ -28,42 +26,13 @@ fn test_primitive<T>(value: T)
 where
     T: Copy + Serialize + Into<JsValue> + DeserializeOwned + PartialEq + Debug,
 {
-    test(value, value);
-}
-
-fn recurse_and_replace_maps(val: JsValue) -> Option<JsValue> {
-    if val.is_object() {
-        let obj = if js_sys::Map::instanceof(&val) {
-            js_sys::Object::from_entries(&js_sys::Array::from(&val)).unwrap()
-        } else {
-            val.unchecked_into()
-        };
-
-        for key in js_sys::Object::keys(&obj).values() {
-            let key = key.unwrap();
-            let val = Reflect::get(&obj, &key).unwrap();
-
-            if let Some(replacement) = recurse_and_replace_maps(val) {
-                Reflect::set(&obj, &key, &replacement).unwrap();
-            }
-        }
-
-        Some(JsValue::from(obj))
-    } else {
-        None
-    }
+    test_via_into(value, value);
 }
 
 fn assert_json<R>(lhs_value: JsValue, rhs: R)
 where
     R: Serialize + DeserializeOwned + PartialEq + Debug,
 {
-    let lhs_value = if let Some(replacement) = recurse_and_replace_maps(lhs_value.clone()) {
-        replacement
-    } else {
-        lhs_value
-    };
-
     if lhs_value.is_undefined() {
         assert_eq!("null", serde_json::to_string(&rhs).unwrap())
     } else {
@@ -77,11 +46,18 @@ where
     assert_eq!(restored_lhs, rhs, "from_value from {:?}", lhs_value);
 }
 
+fn test_via_json_with_config<T>(value: T, serializer: Serializer)
+where
+    T: Serialize + DeserializeOwned + PartialEq + Debug,
+{
+    assert_json(value.serialize(&serializer).unwrap(), value);
+}
+
 fn test_via_json<T>(value: T)
 where
     T: Serialize + DeserializeOwned + PartialEq + Debug,
 {
-    assert_json(to_value(&value).unwrap(), value);
+    test_via_json_with_config(value, Serializer::new());
 }
 
 macro_rules! test_unsigned {
@@ -136,20 +112,20 @@ macro_rules! test_enum {
             a: "struct content".to_string(),
             b: 42,
         });
-        test_via_json($name::Map::<String, i32>(
-            vec![
-                ("a".to_string(), 12),
-                ("abc".to_string(), -1161),
-                ("b".to_string(), 64)
-            ].into_iter().collect()
-        ));
+        test_via_json_with_config($name::Map::<String, i32>(
+            btreemap!{
+                "a".to_string() => 12,
+                "abc".to_string() => -1161,
+                "b".to_string() => 64,
+            }
+        ), Serializer::new().serialize_maps_as_objects(true));
         test_via_json($name::Seq::<i32, i32> { seq: vec![5, 63, 0, -62, 6] });
     }};
 }
 
 #[wasm_bindgen_test]
 fn unit() {
-    test((), JsValue::UNDEFINED);
+    test_via_into((), JsValue::UNDEFINED);
 }
 
 #[wasm_bindgen_test]
@@ -172,11 +148,11 @@ fn numbers() {
     {
         const MAX_SAFE_INTEGER: i64 = 9_007_199_254_740_991;
 
-        test(0_i64, 0_f64);
-        test(42_i64, 42_f64);
-        test(-42_i64, -42_f64);
-        test(MAX_SAFE_INTEGER, MAX_SAFE_INTEGER as f64);
-        test(-MAX_SAFE_INTEGER, -MAX_SAFE_INTEGER as f64);
+        test_via_into(0_i64, 0_f64);
+        test_via_into(42_i64, 42_f64);
+        test_via_into(-42_i64, -42_f64);
+        test_via_into(MAX_SAFE_INTEGER, MAX_SAFE_INTEGER as f64);
+        test_via_into(-MAX_SAFE_INTEGER, -MAX_SAFE_INTEGER as f64);
         to_value(&(MAX_SAFE_INTEGER + 1)).unwrap_err();
         to_value(&-(MAX_SAFE_INTEGER + 1)).unwrap_err();
         to_value(&std::i64::MIN).unwrap_err();
@@ -186,9 +162,9 @@ fn numbers() {
     {
         const MAX_SAFE_INTEGER: u64 = 9_007_199_254_740_991;
 
-        test(0_u64, 0_f64);
-        test(42_u64, 42_f64);
-        test(MAX_SAFE_INTEGER, MAX_SAFE_INTEGER as f64);
+        test_via_into(0_u64, 0_f64);
+        test_via_into(42_u64, 42_f64);
+        test_via_into(MAX_SAFE_INTEGER, MAX_SAFE_INTEGER as f64);
         to_value(&(MAX_SAFE_INTEGER + 1)).unwrap_err();
         to_value(&std::u64::MAX).unwrap_err();
     }
@@ -200,7 +176,7 @@ fn numbers() {
 #[wasm_bindgen_test]
 fn strings() {
     fn test_str(s: &'static str) {
-        let value = to_value(&s).unwrap();
+        let value = to_value(s).unwrap();
         assert_eq!(value, s);
         let restored: String = from_value(value).unwrap();
         assert_eq!(s, restored);
@@ -214,9 +190,9 @@ fn strings() {
 
 #[wasm_bindgen_test]
 fn chars() {
-    test('a', "a");
-    test('\0', "\0");
-    test('😃', "😃");
+    test_via_into('a', "a");
+    test_via_into('\0', "\0");
+    test_via_into('😃', "😃");
 }
 
 #[wasm_bindgen_test]
@@ -245,13 +221,13 @@ fn bytes() {
 
 #[wasm_bindgen_test]
 fn options() {
-    test(Some(0_u32), 0_u32);
-    test(Some(32_u32), 32_u32);
-    test(None::<u32>, JsValue::UNDEFINED);
+    test_via_into(Some(0_u32), 0_u32);
+    test_via_into(Some(32_u32), 32_u32);
+    test_via_into(None::<u32>, JsValue::UNDEFINED);
 
-    test(Some("".to_string()), "");
-    test(Some("abc".to_string()), "abc");
-    test(None::<String>, JsValue::UNDEFINED);
+    test_via_into(Some("".to_string()), "");
+    test_via_into(Some("abc".to_string()), "abc");
+    test_via_into(None::<String>, JsValue::UNDEFINED);
 
     // This one is an unfortunate edge case that won't roundtrip,
     // but it's pretty unlikely in real-world code.
@@ -295,20 +271,13 @@ fn enums() {
     // Internal tags with maps are not properly deserialized from Map values due to the exclusion
     // of Iterables during deserialize_any(). They can be deserialized properly from plain objects
     // so we can test that.
-    assert_eq!(
-        InternallyTagged::Map(
-            vec![
-                ("a".to_string(), 12),
-                ("abc".to_string(), -1161),
-                ("b".to_string(), 64)
-            ]
-            .into_iter()
-            .collect()
-        ),
-        from_value::<InternallyTagged<String, i32>>(
-            js_sys::eval("({ 'tag': 'Map', 'a': 12, 'abc': -1161, 'b': 64 })").unwrap()
-        )
-        .unwrap()
+    test_via_json_with_config(
+        InternallyTagged::Map(btreemap! {
+            "a".to_string() => 12,
+            "abc".to_string() => -1161,
+            "b".to_string() => 64,
+        }),
+        Serializer::new().serialize_maps_as_objects(true),
     );
 
     test_enum! {
@@ -326,7 +295,7 @@ fn structs() {
     #[derive(Debug, PartialEq, Serialize, Deserialize)]
     struct Unit;
 
-    test(Unit, JsValue::UNDEFINED);
+    test_via_into(Unit, JsValue::UNDEFINED);
 
     #[derive(Debug, PartialEq, Serialize, Deserialize)]
     struct Newtype<A>(A);
@@ -358,7 +327,7 @@ fn sequences() {
 
     // Sets are currently indistinguishable from other sequences for
     // Serde serialisers, so this will become an array on the JS side.
-    test_via_json::<HashSet<bool>>([false, true].iter().cloned().collect());
+    test_via_json(hashset! {false, true});
 }
 
 #[wasm_bindgen_test]
@@ -371,29 +340,22 @@ fn maps() {
 
     // Create a Rust HashMap with non-string keys to make sure
     // that we support real arbitrary maps.
-    let mut src = HashMap::new();
-
-    src.insert(
+    let src = hashmap! {
         Struct {
             a: 1,
             b: "smth".to_string(),
-        },
-        Struct {
+        } => Struct {
             a: 2,
             b: "SMTH".to_string(),
         },
-    );
-
-    src.insert(
         Struct {
             a: 42,
             b: "something".to_string(),
-        },
-        Struct {
+        } => Struct {
             a: 84,
             b: "SOMETHING".to_string(),
         },
-    );
+    };
 
     // Convert to a JS value
     let res = to_value(&src).unwrap();
@@ -420,30 +382,18 @@ fn maps_objects_string_key() {
         b: B,
     }
 
-    let serializer = Serializer::new().serialize_maps_as_objects(true);
-
-    let mut src = HashMap::new();
-    src.insert(
-        "a".to_string(),
-        Struct {
+    let src = hashmap! {
+        "a".to_string() => Struct {
             a: 2,
             b: "S".to_string(),
         },
-    );
-    src.insert(
-        "b".to_string(),
-        Struct {
+        "b".to_string() => Struct {
             a: 3,
             b: "T".to_string(),
         },
-    );
+    };
 
-    let res = src.serialize(&serializer).unwrap();
-
-    let res = res.dyn_into::<js_sys::Object>().unwrap();
-    assert_eq!(js_sys::Object::entries(&res).length() as usize, src.len());
-
-    assert_json(res.into(), src);
+    test_via_json_with_config(src, Serializer::new().serialize_maps_as_objects(true));
 }
 
 #[wasm_bindgen_test]
@@ -456,28 +406,22 @@ fn maps_objects_object_key() {
 
     let serializer = Serializer::new().serialize_maps_as_objects(true);
 
-    let mut src = HashMap::new();
-    src.insert(
+    let src = hashmap! {
         Struct {
             a: 1,
             b: "smth".to_string(),
-        },
-        Struct {
+        } => Struct {
             a: 2,
             b: "SMTH".to_string(),
         },
-    );
-
-    src.insert(
         Struct {
             a: 42,
             b: "something".to_string(),
-        },
-        Struct {
+        } => Struct {
             a: 84,
             b: "SOMETHING".to_string(),
         },
-    );
+    };
 
     let res = src.serialize(&serializer).unwrap_err();
     assert_eq!(
